@@ -1,4 +1,4 @@
-{- 
+{-
     Copyright 2013-2019 Mario Blazevic
 
     License: BSD3 (see BSD3-LICENSE.txt file)
@@ -70,7 +70,7 @@ import qualified Data.Monoid.Instances.Positioned as Positioned
 import Data.Semigroup (Semigroup, (<>), Max, Min)
 import Data.Monoid (Monoid, mempty, mconcat, All(All), Any(Any), Dual(Dual),
                     First(First), Last(Last), Sum(Sum), Product(Product))
-import Data.Semigroup.Factorial (Factorial, StableFactorial, 
+import Data.Semigroup.Factorial (Factorial, StableFactorial,
                                  factors, primePrefix, primeSuffix, foldl, foldl', foldr, length, reverse)
 import Data.Semigroup.Cancellative (Commutative, Reductive,
                                     LeftReductive, RightReductive,
@@ -231,6 +231,8 @@ factorialInstances :: [FactorialMonoidInstance]
 factorialInstances = map upcast stableFactorialInstances
                      ++ [FactorialMonoidInstance (mempty :: Sum Integer),
                          FactorialMonoidInstance (mempty :: Product Int32),
+                         FactorialMonoidInstance (mempty :: Identity String),
+                         FactorialMonoidInstance (mempty :: Const String Int),
                          FactorialMonoidInstance (mempty :: Maybe String),
                          FactorialMonoidInstance (mempty :: (Text, String)),
                          FactorialMonoidInstance (mempty :: (Product Int32, ByteString, Sum Integer)),
@@ -247,7 +249,7 @@ factorialInstances = map upcast stableFactorialInstances
    where upcast (StableFactorialMonoidInstance i) = FactorialMonoidInstance i
 
 stableFactorialInstances :: [StableFactorialMonoidInstance]
-stableFactorialInstances = stable1 ++ map measure stable1 ++ map prefixed stable1 ++ map position stable1 
+stableFactorialInstances = stable1 ++ map measure stable1 ++ map prefixed stable1 ++ map position stable1
    where stable1 = map upcast stableTextualInstances
                    ++ [StableFactorialMonoidInstance (mempty :: ByteString),
                        StableFactorialMonoidInstance (mempty :: Lazy.ByteString),
@@ -257,7 +259,7 @@ stableFactorialInstances = stable1 ++ map measure stable1 ++ map prefixed stable
          upcast (StableTextualMonoidInstance i) = StableFactorialMonoidInstance i
          measure (StableFactorialMonoidInstance i) = StableFactorialMonoidInstance (Measured.measure i)
          prefixed (StableFactorialMonoidInstance i) = StableFactorialMonoidInstance (PrefixMemory.shadowed i)
-         position (StableFactorialMonoidInstance (i :: a)) = 
+         position (StableFactorialMonoidInstance (i :: a)) =
             StableFactorialMonoidInstance (pure i :: OffsetPositioned a)
 
 textualInstances :: [TextualMonoidInstance]
@@ -282,7 +284,7 @@ stableTextualInstances = stable1 ++ map measure stable1 ++ map prefixed stable1
                     StableTextualMonoidInstance (mempty :: Vector Char)]
          measure (StableTextualMonoidInstance i) = StableTextualMonoidInstance (Measured.measure i)
          prefixed (StableTextualMonoidInstance i) = StableTextualMonoidInstance (PrefixMemory.shadowed i)
-         position (StableTextualMonoidInstance (i :: a)) = 
+         position (StableTextualMonoidInstance (i :: a)) =
             [StableTextualMonoidInstance (pure i :: OffsetPositioned a),
              StableTextualMonoidInstance (pure i :: LinePositioned a)]
 
@@ -338,6 +340,8 @@ reductiveInstances = map upcast cancellativeInstances
 
 overlappingGCDMonoidInstances = map upcast monusInstances
                                ++ [OverlappingGCDMonoidInstance (mempty :: String),
+                                   OverlappingGCDMonoidInstance (mempty :: Identity String),
+                                   OverlappingGCDMonoidInstance (mempty :: Const String Int),
                                    OverlappingGCDMonoidInstance (mempty :: Seq Int),
                                    OverlappingGCDMonoidInstance (mempty :: ByteString),
                                    OverlappingGCDMonoidInstance (mempty :: Lazy.ByteString),
@@ -350,12 +354,19 @@ overlappingGCDMonoidInstances = map upcast monusInstances
 
 monusInstances = [MonusInstance (mempty :: Product Natural),
                   MonusInstance (mempty :: Sum Natural),
+                  MonusInstance (mempty :: Identity (Sum Natural)),
+                  MonusInstance (mempty :: Const (Sum Natural) Int),
                   MonusInstance (mempty :: Dual (Product Natural)),
                   MonusInstance (mempty :: Maybe ()),
                   MonusInstance (mempty :: Maybe (Product Natural)),
                   MonusInstance (mempty :: Maybe (Sum Natural)),
                   MonusInstance (mempty :: IntSet),
-                  MonusInstance (mempty :: Set String)]
+                  MonusInstance (mempty :: Set String),
+                  MonusInstance (mempty :: ()),
+                  MonusInstance (mempty :: (Sum Natural)),
+                  MonusInstance (mempty :: (Sum Natural, Sum Natural)),
+                  MonusInstance (mempty :: (Sum Natural, Sum Natural, Sum Natural)),
+                  MonusInstance (mempty :: (Sum Natural, Sum Natural, Sum Natural, Sum Natural))]
 
 leftCancellativeInstances = map upcast cancellativeInstances
                             ++ [LeftCancellativeMonoidInstance (mempty :: String),
@@ -388,6 +399,8 @@ leftGCDInstances = map upcast gcdInstances
                        LeftGCDMonoidInstance (mempty :: Lazy.ByteString),
                        LeftGCDMonoidInstance (mempty :: Text),
                        LeftGCDMonoidInstance (mempty :: Lazy.Text),
+                       LeftGCDMonoidInstance (mempty :: Identity ByteString),
+                       LeftGCDMonoidInstance (mempty :: Const ByteString Int),
                        LeftGCDMonoidInstance (mempty :: Dual ByteString),
                        LeftGCDMonoidInstance (mempty :: (Text, String)),
                        LeftGCDMonoidInstance (mempty :: (ByteString, Text, String)),
@@ -410,6 +423,8 @@ rightGCDInstances = map upcast gcdInstances
                        RightGCDMonoidInstance (mempty :: Text),
                        RightGCDMonoidInstance (mempty :: Lazy.Text),
                        RightGCDMonoidInstance (mempty :: String),
+                       RightGCDMonoidInstance (mempty :: Identity ByteString),
+                       RightGCDMonoidInstance (mempty :: Const ByteString Int),
                        RightGCDMonoidInstance (mempty :: Dual String),
                        RightGCDMonoidInstance (mempty :: (Seq Int, ByteString)),
                        RightGCDMonoidInstance (mempty :: Seq Int),
@@ -422,6 +437,8 @@ rightGCDInstances = map upcast gcdInstances
 gcdInstances =
     [ GCDMonoidInstance (mempty :: ())
     , GCDMonoidInstance (mempty :: Product Natural)
+    , GCDMonoidInstance (mempty :: Identity (Sum Natural))
+    , GCDMonoidInstance (mempty :: Const (Sum Natural) Int)
     , GCDMonoidInstance (mempty :: Dual (Product Natural))
     , GCDMonoidInstance (mempty :: IntSet)
     , GCDMonoidInstance (mempty :: Set String)
@@ -436,6 +453,8 @@ distributiveGCDMonoidInstances =
     , DistributiveGCDMonoidInstance (mempty :: Set ())
     , DistributiveGCDMonoidInstance (mempty :: Set Bool)
     , DistributiveGCDMonoidInstance (mempty :: Set Word)
+    , DistributiveGCDMonoidInstance (mempty :: Identity (Sum Natural))
+    , DistributiveGCDMonoidInstance (mempty :: Const (Sum Natural) Int)
     , DistributiveGCDMonoidInstance (mempty :: Dual (Set ()))
     , DistributiveGCDMonoidInstance (mempty :: Dual (Set Bool))
     , DistributiveGCDMonoidInstance (mempty :: Dual (Set Word))
@@ -466,6 +485,12 @@ leftDistributiveGCDMonoidInstances =
     , LeftDistributiveGCDMonoidInstance (mempty :: Set Bool)
     , LeftDistributiveGCDMonoidInstance (mempty :: Set Word)
       -- Instances for monoid transformers:
+    , LeftDistributiveGCDMonoidInstance (mempty :: Identity [()])
+    , LeftDistributiveGCDMonoidInstance (mempty :: Identity [Bool])
+    , LeftDistributiveGCDMonoidInstance (mempty :: Identity [Word])
+    , LeftDistributiveGCDMonoidInstance (mempty :: Const [()] Int)
+    , LeftDistributiveGCDMonoidInstance (mempty :: Const [Bool] Int)
+    , LeftDistributiveGCDMonoidInstance (mempty :: Const [Word] Int)
     , LeftDistributiveGCDMonoidInstance (mempty :: Dual [()])
     , LeftDistributiveGCDMonoidInstance (mempty :: Dual [Bool])
     , LeftDistributiveGCDMonoidInstance (mempty :: Dual [Word])
@@ -496,6 +521,12 @@ rightDistributiveGCDMonoidInstances =
     , RightDistributiveGCDMonoidInstance (mempty :: Set Bool)
     , RightDistributiveGCDMonoidInstance (mempty :: Set Word)
       -- Instances for monoid transformers:
+    , RightDistributiveGCDMonoidInstance (mempty :: Identity [()])
+    , RightDistributiveGCDMonoidInstance (mempty :: Identity [Bool])
+    , RightDistributiveGCDMonoidInstance (mempty :: Identity [Word])
+    , RightDistributiveGCDMonoidInstance (mempty :: Const [()] Int)
+    , RightDistributiveGCDMonoidInstance (mempty :: Const [Bool] Int)
+    , RightDistributiveGCDMonoidInstance (mempty :: Const [Word] Int)
     , RightDistributiveGCDMonoidInstance (mempty :: Dual [()])
     , RightDistributiveGCDMonoidInstance (mempty :: Dual [Bool])
     , RightDistributiveGCDMonoidInstance (mempty :: Dual [Word])
@@ -504,6 +535,8 @@ rightDistributiveGCDMonoidInstances =
 lcmInstances =
     [LCMMonoidInstance (mempty :: Product Natural),
      LCMMonoidInstance (mempty :: Sum Natural),
+     LCMMonoidInstance (mempty :: Identity (Sum Natural)),
+     LCMMonoidInstance (mempty :: Const (Sum Natural) Int),
      LCMMonoidInstance (mempty :: Dual (Product Natural)),
      LCMMonoidInstance (mempty :: Dual (Sum Natural)),
      LCMMonoidInstance (mempty :: IntSet),
@@ -525,6 +558,8 @@ distributiveLCMInstances =
     , DistributiveLCMMonoidInstance (mempty :: Set ())
     , DistributiveLCMMonoidInstance (mempty :: Set Bool)
     , DistributiveLCMMonoidInstance (mempty :: Set Word)
+    , DistributiveLCMMonoidInstance (mempty :: Identity (Sum Natural))
+    , DistributiveLCMMonoidInstance (mempty :: Const (Sum Natural) Int)
     , DistributiveLCMMonoidInstance (mempty :: Dual (Product Natural))
     , DistributiveLCMMonoidInstance (mempty :: Dual (Sum Natural))
     ]
@@ -542,14 +577,14 @@ checkInstances (TextualTest checkType) = (map checkType textualInstances)
 checkInstances (LeftReductiveTest checkType) = (map checkType leftReductiveInstances)
 checkInstances (RightReductiveTest checkType) = (map checkType rightReductiveInstances)
 checkInstances (ReductiveTest checkType) = (map checkType reductiveInstances)
-checkInstances (LeftCancellativeTest checkType) = (map checkType leftCancellativeInstances) 
-checkInstances (RightCancellativeTest checkType) = (map checkType rightCancellativeInstances) 
-checkInstances (CancellativeTest checkType) = (map checkType cancellativeInstances) 
+checkInstances (LeftCancellativeTest checkType) = (map checkType leftCancellativeInstances)
+checkInstances (RightCancellativeTest checkType) = (map checkType rightCancellativeInstances)
+checkInstances (CancellativeTest checkType) = (map checkType cancellativeInstances)
 checkInstances (OverlappingGCDTest checkType) = (map checkType overlappingGCDMonoidInstances)
 checkInstances (MonusTest checkType) = (map checkType monusInstances)
-checkInstances (LeftGCDTest checkType) = (map checkType leftGCDInstances) 
-checkInstances (RightGCDTest checkType) = (map checkType rightGCDInstances) 
-checkInstances (GCDTest checkType) = (map checkType gcdInstances)  
+checkInstances (LeftGCDTest checkType) = (map checkType leftGCDInstances)
+checkInstances (RightGCDTest checkType) = (map checkType rightGCDInstances)
+checkInstances (GCDTest checkType) = (map checkType gcdInstances)
 checkInstances (DistributiveGCDTest checkType) = (map checkType distributiveGCDMonoidInstances)
 checkInstances (LeftDistributiveGCDTest checkType) = (map checkType leftDistributiveGCDMonoidInstances)
 checkInstances (RightDistributiveGCDTest checkType) = (map checkType rightDistributiveGCDMonoidInstances)
@@ -629,6 +664,12 @@ tests = [("CommutativeMonoid", CommutativeTest checkCommutative),
          ("overlap idempotence", OverlappingGCDTest checkOverlap_idempotence),
          ("overlap identity (left)", OverlappingGCDTest checkOverlap_identity_left),
          ("overlap identity (right)", OverlappingGCDTest checkOverlap_identity_right),
+         ("Monus identity (1)", MonusTest checkMonus_identity_1),
+         ("Monus identity (2)", MonusTest checkMonus_identity_2),
+         ("Monus mappend (1)", MonusTest checkMonus_mappend_1),
+         ("Monus mappend (2)", MonusTest checkMonus_mappend_2),
+         ("Monus stripPrefixOverlap", MonusTest checkMonus_stripPrefixOverlap),
+         ("Monus stripSuffixOverlap", MonusTest checkMonus_stripSuffixOverlap),
          ("isPrefixOf", LeftReductiveTest checkIsPrefixOf),
          ("stripSuffix", RightReductiveTest checkStripSuffix),
          ("isSuffixOf", RightReductiveTest checkIsSuffixOf),
@@ -694,16 +735,16 @@ checkConcatFactors (FactorialMonoidInstance (e :: a)) = null (factors e) .&&. fo
 checkFactorsOfFactors (FactorialMonoidInstance (_ :: a)) = forAll (arbitrary :: Gen a) (all singleton . factors)
    where singleton prime = factors prime == [prime]
 
-checkSplitPrimePrefix (FactorialMonoidInstance (_ :: a)) = 
+checkSplitPrimePrefix (FactorialMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) (\a-> factors a == unfoldr splitPrimePrefix a)
 
 checkSplitPrimeSuffix (FactorialMonoidInstance (_ :: a)) = forAll (arbitrary :: Gen a) check
    where check a = factors a == reverse (unfoldr (fmap swap . splitPrimeSuffix) a)
 
-checkPrimePrefix (FactorialMonoidInstance (_ :: a)) = 
+checkPrimePrefix (FactorialMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) (\a-> primePrefix a == maybe mempty fst (splitPrimePrefix a))
 
-checkPrimeSuffix (FactorialMonoidInstance (_ :: a)) = 
+checkPrimeSuffix (FactorialMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) (\a-> primeSuffix a == maybe mempty snd (splitPrimeSuffix a))
 
 checkInits (FactorialMonoidInstance (_ :: a)) =
@@ -712,16 +753,16 @@ checkInits (FactorialMonoidInstance (_ :: a)) =
 checkTails (FactorialMonoidInstance (_ :: a)) =
    mapSize (`div` 5) $ forAll (arbitrary :: Gen a) (\a-> tails a == List.map mconcat (List.tails $ factors a))
 
-checkLeftFold (FactorialMonoidInstance (_ :: a)) = 
+checkLeftFold (FactorialMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) (\a-> foldl (flip (:)) [] a == List.foldl (flip (:)) [] (factors a))
 
-checkLeftFold' (FactorialMonoidInstance (_ :: a)) = 
+checkLeftFold' (FactorialMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) (\a-> foldl' (flip (:)) [] a == List.foldl' (flip (:)) [] (factors a))
 
-checkRightFold (FactorialMonoidInstance (_ :: a)) = 
+checkRightFold (FactorialMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) (\a-> foldr (:) [] a == List.foldr (:) [] (factors a))
 
-checkLength (FactorialMonoidInstance (_ :: a)) = 
+checkLength (FactorialMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) (\a-> length a == List.length (factors a))
 
 checkSpan (FactorialMonoidInstance (_ :: a)) = property $ \p-> forAll (arbitrary :: Gen a) (check p)
@@ -744,19 +785,19 @@ checkSplitAt (FactorialMonoidInstance (_ :: a)) = property $ \i-> forAll (arbitr
    where check i a = splitAt i a == (mconcat l, mconcat r)
             where (l, r) = List.splitAt i (factors a)
 
-checkReverse (FactorialMonoidInstance (_ :: a)) = 
+checkReverse (FactorialMonoidInstance (_ :: a)) =
    property $ forAll (arbitrary :: Gen a) (\a-> reverse a == mconcat (List.reverse $ factors a))
 
 checkStability (StableFactorialMonoidInstance (_ :: a)) =
    property $ forAll (arbitrary :: Gen (a, a)) (\(a, b)-> factors (a <> b) == factors a <> factors b)
 
-checkFromText (TextualMonoidInstance (_ :: a)) = 
+checkFromText (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen Text) (\t-> Textual.fromText t == (fromString (Text.unpack t) :: a))
 
-checkSingleton (TextualMonoidInstance (_ :: a)) = 
+checkSingleton (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen Char) (\c-> Textual.singleton c == (fromString [c] :: a))
 
-checkSplitCharacterPrefix (TextualMonoidInstance (_ :: a)) = 
+checkSplitCharacterPrefix (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen String) check1 .&&. forAll (arbitrary :: Gen a) check2
    where check1 s = unfoldr Textual.splitCharacterPrefix (fromString s :: a) == s
          check2 t = Textual.splitCharacterPrefix (primePrefix t)
@@ -774,7 +815,7 @@ checkUnfoldrToFactors (TextualMonoidInstance (_ :: a)) = forAll (arbitrary :: Ge
 checkFactorsFromString (TextualMonoidInstance (_ :: a)) = forAll (arbitrary :: Gen String) check
    where check s = unfoldr Textual.splitCharacterPrefix (fromString s :: a) == s
 
-checkTextualMap (TextualMonoidInstance (_ :: a)) = 
+checkTextualMap (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = Textual.map wrapSucc a == Textual.concatMap (Textual.singleton . wrapSucc) a
                     && Textual.map id a == a
@@ -783,7 +824,7 @@ checkTextualMap (TextualMonoidInstance (_ :: a)) =
             | c == maxBound = minBound
             | otherwise = succ c
 
-checkConcatMap (TextualMonoidInstance (_ :: a)) = 
+checkConcatMap (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = Textual.concatMap (fromString . f) a == mconcat (map apply $ factors a)
                     && Textual.concatMap Textual.singleton a == a
@@ -797,19 +838,19 @@ checkAll (TextualMonoidInstance (_ :: a)) = forAll (arbitrary :: Gen a) check
 checkAny (TextualMonoidInstance (_ :: a)) = forAll (arbitrary :: Gen a) check
    where check a = Textual.any isLetter a == Textual.foldr (const id) ((||) . isLetter) False a
 
-checkTextualFoldl (TextualMonoidInstance (_ :: a)) = 
+checkTextualFoldl (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = Textual.foldl (\l a-> Left a : l) (\l c-> Right c : l) [] a == List.reverse (textualFactors a)
                     && Textual.foldl (<>) (\a-> (a <>) . Textual.singleton) mempty a == a
          check2 s = Textual.foldl undefined (flip (:)) [] s == List.foldl (flip (:)) [] s
 
-checkTextualFoldr (TextualMonoidInstance (_ :: a)) = 
+checkTextualFoldr (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = Textual.foldr (\a l-> Left a : l) (\c l-> Right c : l) [] a == textualFactors a
                     && Textual.foldr (<>) ((<>) . Textual.singleton) mempty a == a
          check2 s = Textual.foldr undefined (:) [] (fromString s :: a) == s
 
-checkTextualFoldl' (TextualMonoidInstance (_ :: a)) = 
+checkTextualFoldl' (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = Textual.foldl' (\l a-> Left a : l) (\l c-> Right c : l) [] a == List.reverse (textualFactors a)
                     && Textual.foldl' (<>) (\a-> (a <>) . Textual.singleton) mempty a == a
@@ -868,28 +909,28 @@ checkToText (TextualMonoidInstance (_ :: a)) =
    where check1 a = forAll arbitrary $ \f-> Textual.toText f a == Textual.foldr (\t s-> f t <> s) Text.cons Text.empty a
          check2 s = Textual.toText undefined (Textual.fromText s :: a) == s
 
-checkTextualMapAccumL (TextualMonoidInstance (_ :: a)) = 
+checkTextualMapAccumL (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = uncurry (Textual.mapAccumL (,)) ((), a) == ((), a)
          check2 s = Textual.mapAccumL f c (fromString s :: a) == fmap fromString (List.mapAccumL f c s)
          c = 0 :: Int
          f n c = if isLetter c then (succ n, succ c) else (2*n, c)
 
-checkTextualMapAccumR (TextualMonoidInstance (_ :: a)) = 
+checkTextualMapAccumR (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = uncurry (Textual.mapAccumR (,)) ((), a) == ((), a)
          check2 s = Textual.mapAccumR f c (fromString s :: a) == fmap fromString (List.mapAccumR f c s)
          c = 0 :: Int
          f n c = if isLetter c then (succ n, succ c) else (2*n, c)
 
-checkTextualTakeWhile (TextualMonoidInstance (_ :: a)) = 
+checkTextualTakeWhile (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = textualFactors (Textual.takeWhile (const True) isLetter a)
                     == List.takeWhile (either (const True) isLetter) (textualFactors a)
                     && Textual.takeWhile (const True) (const True) a == a
          check2 s = Textual.takeWhile undefined isLetter (fromString s :: a) == fromString (List.takeWhile isLetter s)
 
-checkTextualDropWhile (TextualMonoidInstance (_ :: a)) = 
+checkTextualDropWhile (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = textualFactors (Textual.dropWhile (const True) isLetter a)
                     == List.dropWhile (either (const True) isLetter) (textualFactors a)
@@ -942,14 +983,14 @@ checkTextualSpanMaybe_' (TextualMonoidInstance (_ :: a)) =
                   foldMaybe = Textual.foldl_' gc (Just s0)
                   gc s c = s >>= flip fc c
 
-checkTextualTakeWhile_ (TextualMonoidInstance (_ :: a)) = 
+checkTextualTakeWhile_ (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = textualFactors (Textual.takeWhile_ True isLetter a)
                     == List.takeWhile (either (const True) isLetter) (textualFactors a)
                     && Textual.takeWhile_ True (const True) a == a
          check2 s = Textual.takeWhile_ undefined isLetter (fromString s :: a) == fromString (List.takeWhile isLetter s)
 
-checkTextualDropWhile_ (TextualMonoidInstance (_ :: a)) = 
+checkTextualDropWhile_ (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = textualFactors (Textual.dropWhile_ True isLetter a)
                     == List.dropWhile (either (const True) isLetter) (textualFactors a)
@@ -961,7 +1002,7 @@ checkTextualSplit (TextualMonoidInstance (_ :: a)) = forAll (arbitrary :: Gen a)
    where check a = List.all (List.all isLetter . rights . textualFactors) (Textual.split (not . isLetter) a)
                    && (mconcat . intersperse (fromString " ") . Textual.split (== ' ')) a == a
 
-checkTextualFind (TextualMonoidInstance (_ :: a)) = 
+checkTextualFind (TextualMonoidInstance (_ :: a)) =
    forAll (arbitrary :: Gen a) check1 .&&. forAll (arbitrary :: Gen String) check2
    where check1 a = Textual.find isLetter a == (List.find isLetter . rights . textualFactors) a
          check2 s = Textual.find isLetter (fromString s :: a) == List.find isLetter s
@@ -1263,11 +1304,41 @@ checkLCM_distributivity_lcm_gcd (DistributiveLCMMonoidInstance (_ :: a)) =
   where
     check a b c = gcd a (lcm b c) === lcm (gcd a b) (gcd a c)
 
+checkMonus_identity_1 (MonusInstance (_ :: a)) =
+    forAll (arbitrary :: Gen a) check
+  where
+    check a = a <\> a === mempty
+
+checkMonus_identity_2 (MonusInstance (_ :: a)) =
+    forAll (arbitrary :: Gen a) check
+  where
+    check a = mempty <\> a === mempty
+
+checkMonus_mappend_1 (MonusInstance (_ :: a)) =
+    forAll (arbitrary :: Gen (a, a)) check
+  where
+    check a b = a <> (b <\> a) === b <> (a <\> b)
+
+checkMonus_mappend_2 (MonusInstance (_ :: a)) =
+    forAll (arbitrary :: Gen (a, a, a)) check
+  where
+    check (a, b, c) = (a <\> b) <\> c === a <\> (b <> c)
+
+checkMonus_stripPrefixOverlap (MonusInstance (_ :: a)) =
+    forAll (arbitrary :: Gen (a, a)) check
+  where
+    check (a, b) = (a <\> b) === stripPrefixOverlap b a
+
+checkMonus_stripSuffixOverlap (MonusInstance (_ :: a)) =
+    forAll (arbitrary :: Gen (a, a)) check
+  where
+    check (a, b) = (a <\> b) === stripSuffixOverlap b a
+
 textualFactors :: TextualMonoid t => t -> [Either t Char]
 textualFactors = map characterize . factors
    where characterize prime = maybe (Left prime) Right (Textual.characterPrefix prime)
 
-newtype TestString = TestString String deriving (Eq, Show, Arbitrary, CoArbitrary, 
+newtype TestString = TestString String deriving (Eq, Show, Arbitrary, CoArbitrary,
                                                  Semigroup, LeftReductive, LeftCancellative, StableFactorial,
                                                  Monoid, LeftGCDMonoid,
                                                  MonoidNull, PositiveMonoid, IsString)
